package com.ll.springmvc.servlet;

import com.ll.springmvc.annaotion.*;
import com.ll.springmvc.controller.WriteController;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.lang.annotation.Annotation;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.URL;
import java.util.*;


/**
 * DispatcherServlet
 *
 * @author WangLei
 * Date 2024/6/23 023 20:48
 */
public class DispatcherServlet extends HttpServlet {

    List<String> classNames = new ArrayList<String>();

    Map<String, Object> beans = new HashMap<String, Object>();
    Map<String, Object> harldermap = new HashMap<String, Object>();

    private static final long serialVersionUID = 1L;


    public void init(ServletConfig config) {
        //把所有的bean扫描 ----扫描所有的class文件
        scanPackage("com.ll");

        doInstance(); //根据全类名创建bean

        doIoc();  //根据bean 进行依赖注入

        buildUrlMapping();  //构建url映射

    }


    //com.enjoy
    public void scanPackage(String basePackage) {
        URL url = this.getClass().getClassLoader().getResource("/" + basePackage.replaceAll("\\.", "/"));
        String fileStr = url.getFile();

        File file = new File(fileStr);

        String[] filesStr = file.list();

        for (String path : filesStr) {
            File filePath = new File(fileStr + path);
            if (filePath.isDirectory()) {
                scanPackage(basePackage + "." + path);
            } else {
                //加入LIST
                classNames.add(basePackage + "." + filePath.getName());

            }
        }
    }

    //根据扫描到的List全类名 进行实例化
    public void doInstance() {
        if (classNames.size() <= 0) {
            System.out.println("没有扫描到任何类");
            return;
        }

        //list的所有class
        for (String className : classNames) {
            String cn = className.replace(".class", "");
            try {
                Class<?> aClass = Class.forName(cn);
                if (aClass.isAnnotationPresent(MvcController.class)) {
                    //是controller
                    Object instance = aClass.newInstance(); //创建控制类

                    MvcRequestMapping requestMapping = aClass.getAnnotation(MvcRequestMapping.class);
                    String rmvalue = requestMapping.value();  // /write
                    beans.put(rmvalue, instance);
                } else if (aClass.isAnnotationPresent(MvcService.class)) {
                    //是service
                    Object instance = aClass.newInstance();
                    MvcService service = aClass.getAnnotation(MvcService.class);
                    String serviceName = service.value();
                    beans.put(serviceName, instance);
                } else {
                    continue;
                }
            } catch (ClassNotFoundException e) {
                throw new RuntimeException(e);
            } catch (InstantiationException e) {
                throw new RuntimeException(e);
            } catch (IllegalAccessException e) {
                throw new RuntimeException(e);
            }

        }
    }


    //把service注入到controller
    public void doIoc() {

        if (beans.entrySet().size() <= 0) {
            System.out.println("没有一个实例化类");
        }
        for (Map.Entry<String, Object> entry : beans.entrySet()) {
            Object instance = entry.getValue();
            Class<?> clazz = instance.getClass();

            if (clazz.isAnnotationPresent(MvcController.class)) {
                Field[] fields = clazz.getDeclaredFields();
                for (Field field : fields) {
                    if (field.isAnnotationPresent(MvcAutowired.class)) {
                        MvcAutowired autowired = field.getAnnotation(MvcAutowired.class);
                        String key = autowired.value();  // 获取service的key
                        field.setAccessible(true);
                        try {
                            field.set(instance, beans.get(key));
                        } catch (IllegalAccessException e) {
                            throw new RuntimeException(e);
                        }
                    } else {
                        continue;
                    }
                }
            }
        }

    }


    private void buildUrlMapping() {
        if (beans.entrySet().size() <= 0) {
            System.out.println("没有一个实例化类");
            return;
        }

        for (Map.Entry<String, Object> entry : beans.entrySet()) {
            Object instance = entry.getValue();
            Class<?> clazz = instance.getClass();
            if (clazz.isAnnotationPresent(MvcController.class)) {
                MvcRequestMapping requestMapping = clazz.getAnnotation(MvcRequestMapping.class);
                String classPath = requestMapping.value();  // /write

                Method[] methods = clazz.getMethods();

                for (Method method : methods) {
                    if (method.isAnnotationPresent(MvcRequestMapping.class)) {
                        MvcRequestMapping methodMapping = method.getAnnotation(MvcRequestMapping.class);
                        String methodPath = methodMapping.value();
                        harldermap.put(classPath + methodPath, method); // 修改这里
                    } else {
                        continue;
                    }
                }
            }else{
                continue;
            }
        }

    }


    @Override
    protected void doGet(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        this.doPost(req, resp);
    }

    @Override
    protected void doPost(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {

        //获取请求路径
        String url = req.getRequestURI();
        String context = req.getContextPath();  //write-mvc
        String path = url.replace(context, ""); //write/query



        Method method = (Method) harldermap.get(path);
        if (method == null) {
            throw new RuntimeException("No handler method found for the path: " + path);
        }


        //根据key - /write 到 map去拿
        WriteController instance= (WriteController) beans.get("/"+path.split("/")[1]);

        Object arg[] = hand(req, resp, method);
        try {
            method.invoke(instance, arg);
        } catch (IllegalAccessException e) {
            throw new RuntimeException(e);
        } catch (InvocationTargetException e) {
            throw new RuntimeException(e);
        }

    }


    private static Object[] hand(HttpServletRequest request, HttpServletResponse response, Method method){
        //拿到当前执行的方法有哪些参数
        Class<?>[] paramClazzs = method.getParameterTypes();
        //根据参数的个数 创建数组
        Object[] args = new Object[paramClazzs.length];

        int args_i=0;
        int index=0;

        for (Class<?> paramClazz : paramClazzs) {
            if(ServletRequest.class.isAssignableFrom(paramClazz)){
                args[args_i++] = request;
            }
            if(ServletResponse.class.isAssignableFrom(paramClazz)){
                args[args_i++] = response;
            }
            Annotation[] paramAns = method.getParameterAnnotations()[index];
            if (paramAns.length > 0) {

                for (Annotation paramAn : paramAns) {
                    if(MvcRequestParam.class.isAssignableFrom(paramAn.getClass())){
                      MvcRequestParam requestParam = (MvcRequestParam) paramAn;
                      args[args_i++] = request.getParameter(requestParam.value());
                    }
                }
            }
            index++;
        }
        return args;
    }

}
